package com.karneim.util.collection.regex;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author Ralf Meyer
 * @version 1.0
 */
import java.util.*;
import java.lang.ref.WeakReference;
import com.karneim.util.collection.automaton.*;
import com.karneim.util.collection.set.*;

/**
 * Regular expression based on a minimized deterministic automaton (FSA) and designed as a set of strings.
 * <br>Use this class to manipulate a reular expression through set oprations or automaton methods
 * <br>PatternPro differs from Pattern that the contributed set of strings is mutable through the methods
 * addAll, removeAll and retainAll.
 * <br>Further PaternPro provides access to its PAutomaton through the getAutomaton method.
 * So it is possible to inspect the automaton's states through PAutomaton's methods.
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Bro fr Softwarearchitektur   www.karneim.com</p>
 * @author Ralf Meyer
 * @version 1.0
 */
public class PatternPro extends Pattern {

  WeakReference automatonWrapper = null;
  WeakReference patternWrapper = null;

  protected PatternPro(ISet_char fullSet) {
    super(fullSet);
  }

  protected PatternPro(Automaton_Pattern automaton) {
    super(automaton);
  }

  public PatternPro() {
  }

  public PatternPro(PAutomaton automaton) {
    this.setAutomaton(automaton);
  }

  public PatternPro(Pattern p) {
    this.setPattern(p);
  }

  public PatternPro(String regEx) {
    this.setRegEx(regEx);
  }



  public void setRegEx(String regEx) {
    this.automaton = (Automaton_Pattern)Pattern.get(regEx).clone();
  }

  public void setPattern(Pattern p) {
    this.automaton = (Automaton_Pattern)p.automaton.clone();
    this.automaton.minimize();
  }

  public void setAutomaton(PAutomaton a) {
    this.automatonWrapper = new WeakReference(a);
    this.automaton = (Automaton_Pattern)a.getAutomaton();
  }


  public Pattern getPattern() {
    if (this.patternWrapper==null) {
      Pattern answer = new Pattern((Automaton_Pattern)this.automaton);
      this.patternWrapper = new WeakReference(answer);
      return answer;
    }

    Pattern answer = (Pattern)this.patternWrapper.get();
    if (answer!=null) return answer;

    answer = new Pattern((Automaton_Pattern)this.automaton.clone());
    this.patternWrapper = new WeakReference(answer);
    return answer;
  }

  public PAutomaton getAutomaton() {
    if (this.automatonWrapper==null) {
      PAutomaton answer = new PAutomaton(this.automaton);
      this.automatonWrapper = new WeakReference(answer);
      return answer;
    }

    PAutomaton answer = (PAutomaton)this.automatonWrapper.get();
    if (answer!=null) return answer;

    answer = new PAutomaton(this.automaton);
    this.automatonWrapper = new WeakReference(answer);
    return answer;
  }

  public boolean contains(String s,int offset,int length) {
    if (((Automaton_Pattern)this.automaton).isDeterministic())
      return super.contains(s,offset,length);

    Automaton.State state = ((Automaton_Pattern)this.automaton).getStartState();
    if (state==null) return false;

    Automaton.LinkedSet_State states = this.automaton.newLinkedSet_State(state);
    Automaton.LinkedSet_State newStates = this.automaton.newLinkedSet_State();

    for (;length>0; ++offset, --length) {
      loop: for (Automaton.Wrapper_State w=states.elements; w!=null; w=w.next) {
        if (w.state.isDeterministic()) {
          for (Automaton.State.Transition trans=w.state.transitions; trans!=null; trans=trans.next) {
            if (trans.charSet.contains(s.charAt(offset))) {
              newStates.add(trans.toState);
              continue loop;
            }
          }
        } else {
          for (Automaton.State.Transition trans=w.state.transitions; trans!=null; trans=trans.next) {
            if (trans.charSet.contains(s.charAt(offset))) {
              newStates.add(trans.toState);
            }
          }
        }
      }

      for (Automaton.Wrapper_State w=newStates.elements; w!=null; w=w.next) {
        for (Automaton.State.Transition trans=w.state.eTransitions; trans!=null; trans=trans.next) {
          newStates.add(trans.toState);
        }
      }

      if (newStates.isEmpty()) return false;

      states = newStates;
      newStates.clear();
    }
    return ((Automaton_Pattern.LinkedSet_PState)states).isFinal();
  }

  public boolean contains(char[] chars,int offset,int length) {
    if (((Automaton_Pattern)this.automaton).isDeterministic())
      return super.contains(chars,offset,length);

    Automaton.State state = ((Automaton_Pattern)this.automaton).getStartState();
    if (state==null) return false;

    Automaton.LinkedSet_State states = this.automaton.newLinkedSet_State(state);
    Automaton.LinkedSet_State newStates = this.automaton.newLinkedSet_State();

    for (;length>0; ++offset, --length) {
      loop: for (Automaton.Wrapper_State w=states.elements; w!=null; w=w.next) {
        if (w.state.isDeterministic()) {
          for (Automaton.State.Transition trans=w.state.transitions; trans!=null; trans=trans.next) {
            if (trans.charSet.contains(chars[offset])) {
              newStates.add(trans.toState);
              continue loop;
            }
          }
        } else {
          for (Automaton.State.Transition trans=w.state.transitions; trans!=null; trans=trans.next) {
            if (trans.charSet.contains(chars[offset])) {
              newStates.add(trans.toState);
            }
          }
        }
      }

      for (Automaton.Wrapper_State w=newStates.elements; w!=null; w=w.next) {
        for (Automaton.State.Transition trans=w.state.eTransitions; trans!=null; trans=trans.next) {
          newStates.add(trans.toState);
        }
      }

      if (newStates.isEmpty()) return false;

      states = newStates;
      newStates.clear();
    }
    return ((Automaton_Pattern.LinkedSet_PState)states).isFinal();
  }

  public void complement() {
    this.automaton.complement();
  }

  public void addAll(String regEx) {
    this.automaton.addAll(regEx);
    this.automaton.minimize();
  }

  public void retainAll(String regEx) {
    this.automaton.retainAll(regEx);
    this.automaton.minimize();
  }

  public void removeAll(String regEx) {
    this.automaton.removeAll(regEx);
    this.automaton.minimize();
  }

  public void addAll(Pattern pattern) {
    this.automaton.addAll(pattern.automaton);
    this.automaton.minimize();
  }

  public void retainAll(Pattern pattern) {
    this.automaton.retainAll(pattern.automaton);
    this.automaton.minimize();
  }

  public void removeAll(Pattern pattern) {
    this.automaton.removeAll(pattern.automaton);
    this.automaton.minimize();
  }

  public void addAll(PAutomaton a) {
    this.automaton.addAll((Automaton_Pattern)a.getAutomaton());
    this.automaton.minimize();
  }

  public void retainAll(PAutomaton a) {
    this.automaton.retainAll((Automaton_Pattern)a.getAutomaton());
    this.automaton.minimize();
  }

  public void removeAll(PAutomaton a) {
    this.automaton.removeAll((Automaton_Pattern)a.getAutomaton());
    this.automaton.minimize();
  }

  public void clear() {
    this.automaton.clear();
  }
}